(*  :Title:	Initialization for Object-Oriented Signal Processing Package  *)

(*  :Authors:	Brian Evans, James McClellan  *)

(*  :Summary:	This is an initialization file only  *)

(*  :Context:	Global`  *)

(*  :PackageVersion:  SPP 2.9.5  @(#)ObjectOriented.m	1.2	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*  :History:	*)

(*  :Keywords:	*)

(*  :Source:	*)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)


(*  B E G I N     I N I T I A L I Z A T I O N  *)


(*  Load those routines on which these packages rely  *) 

Block [ {oldoutput, tmpfile, tmpstream},

        If [ TrueQ[ $VersionNumber >= 2.0 ],
             oldoutput = $Output;
             tmpstream = OpenTemporary[];
             $Output = {tmpstream} ];

        Needs[ "SignalProcessing`Support`" ];

        If [ TrueQ[ $VersionNumber >= 2.0 ],
             $Output = oldoutput;
             tmpfile = Close[tmpstream];
             DeleteFile[tmpfile] ] ]


Which [


(*  Check to make sure that these packages has not already been loaded  *)

MemberQ[ $ContextPath, "SignalProcessing`ObjectOriented`" ],

  object::loaded =
	"Version 2.9.5 of the rewrite rules for \
        signals and systems has already been loaded.",


(*  For Mathematica 2.0, we only define the stubs  *)

TrueQ[ $VersionNumber >= 2.0 ],

  DeclarePackage[ "SignalProcessing`ObjectOriented`CostFunctions`", { 
			"AdditiveCosts", "AlgorithmCostFunction",
			"ComputePerOutputSamplingRate", "CostRange",
			"MemoryCosts", "MultiplicationCosts",
			"PerOutputSamplingRate", "SamplingRate" } ];

  DeclarePackage[ "SignalProcessing`ObjectOriented`Heuristics`", { 
			"AllEquivalentForms", "SPHeuristicRewrite",
			"SPRecursiveRewrite" } ];

  DeclarePackage[ "SignalProcessing`ObjectOriented`Ptolemy`", { 
			"BranchOperation", "ExpressionToBranchOperations",
			"FunctionToPtolemy", "InputPortName",
			"OperatorToPtolemy", "OutputPortName",
			"PlusByConstant", "Ptolemy", "PtolemyForm",
			"PtolemyProgram", "$PtolemyProlog",
			"$PtolemyVersion", "TimesByConstant" } ];

  DeclarePackage[ "SignalProcessing`ObjectOriented`RewriteRules`", { 
			"SystemRewriteRules" } ];

  DeclarePackage[ "SignalProcessing`ObjectOriented`Signal`", { 
			"ArraySignal", "ArraySignalQ", "CenterOfSymmetry",
			"ComputableSignalQ", "DiscreteTimeSignalQ",
			"EndBandwidth", "Fetch", "FetchArray", "FetchNext",
			"IIRCoefficients", "IndependentVariable",
			"IIRFilter", "IIRPostUpdate", "MakeContinuousSignal",
			"MakeDigitalSignal", "MakeSignal", "MyMakeSignal",
			"NextFun", "OutOfDomainValue", "PastValues",
			"PointwiseSignal", "PointwiseSignalQ",
			"PostUpdateFunction", "PreUpdateFunction", "PrintName",
			"RealOrComplex", "Self", "Signal", "SignalQ",
			"StartBandwith", "StreamSignal", "StreamSignalQ" } ];

  DeclarePackage[ "SignalProcessing`ObjectOriented`Slot`", { 
			"AddSlot", "AKindOf", "AppendSlot", "DefaultSlot",
			"DefineObjectType", "DeleteSlot", "HasSlotQ",
			"MakeObject", "ReadOnlyList", "ReadOnlySlotQ",
			"ReadSlot", "WriteSlot" } ];

  DeclarePackage[ "SignalProcessing`ObjectOriented`StackQueue`", { 
			"ClearQueue", "ClearStack", "Dequeue", "Enqueue",
			"Pop", "Push", "Queue", "TopElement" } ];

  DeclarePackage[ "SignalProcessing`ObjectOriented`System`", { 
			"ASSOCIATIVE", "ADDITIVE", "CheckSeparability",
			"COMMUTATIVE", "CONTINUOUS", "DefineSystem",
			"DISCRETE", "HOMOGENEOUS", "LINEAR",
			"LINEARPHASE", "MakeSeparable", "MEMORYLESS",
			"MoveOperatorsToFront", "SEPARABLE",
			"SHIFTINVARIANT", "SystemProperties", "SPsystems",
			"SPproperties", "SystemQ", "ValidOperatorQ" } ];

  AppendTo[$ContextPath, "SignalProcessing`ObjectOriented`"];

  object::loaded =
	"Stubs for version 2.9.5 of the routines and rules for \
	defining and manipulating signals and systems as objects \
	have been loaded successfully.",


(*  Object-oriented signal processing --  load in hierarchial order. *)

True,

  Needs[ "SignalProcessing`ObjectOriented`StackQueue`" ];	(* level a *)
  Needs[ "SignalProcessing`ObjectOriented`Slot`" ];
  Needs[ "SignalProcessing`ObjectOriented`Ptolemy`" ];

  Needs[ "SignalProcessing`ObjectOriented`System`" ];		(* level b *)
  Needs[ "SignalProcessing`ObjectOriented`RewriteRules`" ];

  Needs[ "SignalProcessing`ObjectOriented`Signal`" ];		(* level c *)

  AppendTo[$ContextPath, "SignalProcessing`ObjectOriented`"];

  object::loaded =
	"Version 2.9.5 of the routines and rules for \
	defining and manipulating signals and systems \
	as objects have been loaded successfully."

]


(*  Temporary messages  *)

object::error = "The current context should be Global` and it is not."

(*  Error checking  *)

If [ SameQ[$Context, "Global`"],
     Message[ object::loaded ],
     Message[ object::error ] ]

(*  Remove temporary messages  *)

object::error = .
object::loaded = .


(*  E N D     I N I T I A L I Z A T I O N  *)

Null
