(*  :Title:	Initialization for signal processing supporting objects  *)

(*  :Authors:	Brian Evans, James McClellan  *)

(*  :Summary:	This is an initialization file only  *)

(*  :Context:	Global`  *)

(*  :PackageVersion:  SPP 2.9.5  @(#)Support.m	1.2	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*  :History:	*)

(*  :Keywords:	*)

(*  :Source:	*)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)


(*  B E G I N     I N I T I A L I Z A T I O N  *)



Which[


(*  Check to make sure that these have not already been loaded  *)

MemberQ[ $ContextPath, "SignalProcessing`Support`" ],

  support::loaded =
	"Version 2.9.5 of the signal processing support packages has \
	already been loaded",


(*  For Mathematica 2.0, we only define the stubs  *)

TrueQ[ $VersionNumber >= 2.0 ],

  Print["Defining stubs for objects in version 2.9.5 of the \
           signal processing support packages ..."];

  DeclarePackage[ "SignalProcessing`Support`Convolution`", { 
			"$ConvolutionDomain", "Area", "AutoCorrelation",
			"ConvertFromList", "ConvertToList",
			"CTPiecewiseConvolution", "DTPiecewiseConvolution",
			"FormulaToPiecewise", "IntervalQ",
			"PiecewiseConvolution", "PiecewisePlot",
			"PiecewiseToFormula", "PlotList",
			"SetConvolutionDomain", "SimplifyList",
			"SymmetricPulseConvolutions", "ValidIntervalQ",
			"VectorConvolution" } ];

  DeclarePackage[ "SignalProcessing`Support`Decimation`", { 
			"DesignDecimationSystem",
			"DownsamplingAliasing",
			"FactorRationalResamplingMatrix",
			"FindBoundingRegion",
			"FindMinimumRectangle",
			"ValidDecimatorBaseband" } ];

  DeclarePackage[ "SignalProcessing`Support`FilterSupport`", { 
			"Analog", "Bandpass", "Bandstop", "Bessel",
			"BilinearTransformation", "Butterworth", "CFIR",
			"CIIR", "ChebyshevI", "ChebyshevII", "DesignFilter",
			"Digital", "Elliptic", "FilterOrder",
			"FilterParameters", "FIR", "Highpass", "IIR",
			"IIRFunction", "Lowpass" } ];

  DeclarePackage[ "SignalProcessing`Support`LatticeTheory`", { 
			"BezoutNumbers", "ColumnHermiteForm",
			"CommutableResamplingMatricesQ", "DiagonalMatrixQ",
			"DistinctCosetVectors", "EuclidFactors",
			"GCLD", "GCRD", "IncList", "ImpulseTrain",
			"IntegerVectorQ", "LCLM", "LCRM",
			"NormalizeSamplingMatrix", "RegUnimodularResMatrixQ",
			"RelativelyPrime", "ReorderResampling",
			"ResamplingMatrix", "ResamplingMatrixMod",
			"RowHermiteForm", "SmithEqualizedForm",
			"SmithFormMinimizeU", "SmithFormMinimizeV",
			"SmithFormPivot", "SmithFormSameU", "SmithFormSameV",
			"SmithLLLForm", "SmithMcMillanForm",
			"SmithNormalForm", "SmithOrderedForm",
			"SmithReducedForm", "UpsampledSystem" } ];

  DeclarePackage[ "SignalProcessing`Support`Lisp`", { 
			"Adjoin", "ButLast", "ComplexQ", "Cons", "Elt",
			"Every", "Intern", "ListReduce", "Nth", "NthCdr",
			"RationalQ", "RealQ", "Second", "SetExclusiveOr",
			"Some", "SubsetQ", "Third" } ];

  DeclarePackage[ "SignalProcessing`Support`Polygons`", { 
			"ConvexPolygonIntersection", "DistinctRGBColors",
			"DistinctCMYKColors", "InsidePolygonQ",
			"OverlappingRegions", "PolygonArea",
			"PolygonToTriangles", "RearrangePolygon",
			"TriangleModWithSquare" } ];

  DeclarePackage[ "SignalProcessing`Support`ROC`", { 
			"FindRMinus", "FindRPlus", "GetRMinus", "GetRPlus",
			"InfCheck", "IntersectsROC", "LTransData",
			"LVariables", "Rminus", "Rplus", "Transform",
			"WithinROC", "ZTransData", "ZVariables" } ];

  DeclarePackage[ "SignalProcessing`Support`SigProc`", { 
			"$DeltaFunctionScaling", "Aliasby", "CConvolve",
			"CircularShift", "Convolve", "Continuous",
			"CPulse", "CStep", "Delta", "DeltaIntegrate",
			"DeltaPlot", "DFT", "DTFT", "Difference",
			"Dirichlet", "Discrete", "DiscreteGraphics",
			"Domain", "Downsample", "DummyVariables",
			"Extent1D", "FT", "GetDeltaFunctions", "Impulse",
			"Interleave", "IntervalsToFunction", "InvDFT",
			"InvDTFT", "InvFT", "InvL", "InvZ", "L",
			"LineImpulse", "OperatorInOperExpr", "OperatorName",
			"ParametersInOperExpr", "Periodic",
			"PolyphaseDownsample", "PolyphaseResample",
			"PolyphaseUpsample", "Pulse", "RationalGCD", "Rev",
			"RewriteSummations", "ScaleAxis", "ScalingFactor",
			"SequencePlot", "SequenceToFunction", "Shift",
			"SignalCleanup", "SignalPlot", "SignalsInOperExpr",
			"Sinc", "SPSimplify", "Step", "Summation",
			"TheFunction", "ToContinuous", "ToDiscrete",
			"Unit", "Upsample", "UpsampledFunction",
			"UpsampleFactor", "UpsampleSequence", "Z",
			"AliasedSinc", "AliasSinc", "ASinc" } ];

  DeclarePackage[ "SignalProcessing`Support`SmithChart`", { 
			"SmithChart", "SmithChartGraphics",
			"SmithChartPlot" } ];

  DeclarePackage[ "SignalProcessing`Support`SupCode`", { 
			"AllSubsets", "Arrow2D", "AssociateItem", "Assuming",
			"CirclePS", "Combine", "ComplexTo2DCoord",
			"ComplexTo2DCoordList", "ConstantQ", "ConstantTerm",
			"Dialogue", "Element", "EmptyQ", "FindFile",
			"GenerateCode", "GeneratePattern", "GenerateSymbol",
			"GetAllExponents", "GetAllFactors",
			"GetAllShiftFactors", "GetArgs",
			"GetOperatorVariables", "GetRoot", "GetRootList",
			"GetShiftFactor", "GetStateField", "GetVariables",
			"GetValue", "HasAttributes", "ImaginaryQ", "InRange",
			"InfinityQ", "InformUserQ", "ListQ",
			"MixedPolynomialQ", "MyApart", "MyCollectAll",
			"MyFreeQ", "MyMessage", "NullPlot",
			"PartitionIntoSubsets", "PatternQ", "PointwisePlot",
			"PrintIt", "ProtectIt", "QueryTheUser",
			"RationalFunctionQ", "RationalPolynomialQ",
			"ReadTextFile", "RealValuedQ", "RemoveOptions",
			"ReplaceWith", "RuleAppliesQ", "SameFormQ",
			"SetExclusion", "SetStateField", "SPfunctions",
			"SPLessGreaterRules", "SPoperators", "SPsignals",
			"SPSimplificationRules", "StripPackage", "TableLookup",
			"ToCollection", "ToList", "UnprotectIt", "VariableQ",
			"ZeroQ", "ZPolynomial" } ];

  DeclarePackage[ "SignalProcessing`Support`TransSupport`", { 
			"AddT", "ApplyHomogeneityRule", "ConjT", "ConvolveT",
			"DerivativeT", "DomainScale", "IntegrateT", "Linear",
			"LineImpulsemDT", "MagRangeScale", "MagPhasePlot",
			"MultiDInvTransform", "MultiDLookup",
			"MultiDTransform", "MultT", "PhaseRangeScale",
			"PoleZeroPlot", "Radian", "ROCPlot",
			"RootLocus", "ScaleT", "ShadedAnnulus", "Stable",
			"SubT", "SubstituteForT", "TransformFixUp",
			"TransformDialogue", "TransformLookup",
			"MagnitudePhasePlot", "RootLocusPlot" } ];

  DeclarePackage[ "SignalProcessing`Support`Tree`", { 
			"AddChildToTree", "DeleteFromTree",
			"InitTree", "SubTree" } ];

  AppendTo[$ContextPath, "SignalProcessing`Support`"];

  support::loaded =
	"Stubs for the supporting routines, objects, and rules for the \
	signal processing package have been loaded successfully.",


(*  Load necessary files in order of hierarchy, bottom up. *)

True, 

  Print["Loading version 2.9.5 of the signal processing support packages ..."];

  Needs[ "SignalProcessing`Support`SupCode`" ];		(* level a  *)

  Needs[ "SignalProcessing`Support`FilterSupport`" ];	(* level b  *)
  Needs[ "SignalProcessing`Support`SigProc`" ];
  Needs[ "SignalProcessing`Support`LatticeTheory`" ];

  Needs[ "SignalProcessing`Support`ROC`" ];		(* level c  *)
  Needs[ "SignalProcessing`Support`Convolution`" ];

  Needs[ "SignalProcessing`Support`TransSupport`" ];	(* level d  *)

  AppendTo[$ContextPath, "SignalProcessing`Support`"];

  support::loaded =
	"Supporting routines, objects, and rules for the signal \
	processing package have been loaded successfully."

]

(*  Temporary messages  *)

support::error = "The current context should be Global` and it is not."
support::change =
	"The loading process has changed such that the files Tree.m and \
	DataType.m are no longer automatically loaded (the signal \
	processing packages no longer rely on the definitions in these files)."

(*  Error checking  *)

If [ SameQ[$Context, "Global`"],
     Message[ support::loaded ],
     Message[ support::error ] ]

If [ ! TrueQ[ $VersionNumber >= 2.0 ],
     Message[ support::change ] ]

(*  Remove temporary messages  *)

support::error = .
support::loaded = .
support::change = .


(*  E N D     I N I T I A L I Z A T I O N  *)

Null
